/*!
 * module node-red-contrib-stm32cubemonitor
 * Copyright(c) 2019 STMicroelectronics
 */
/* eslint-disable no-console */
/* eslint-disable require-jsdoc */
var stlinkRs = 1;
const dataReceived = {
  data: [
    [
      {
        x: 145,
        y: 110
      }
    ],
    [
      {
        x: 145,
        y: 111
      }
    ],
    [
      {
        x: 145,
        y: 112
      }
    ],
    [
      {
        x: 145,
        y: 113
      }
    ],
    [
      {
        x: 145,
        y: 114
      }
    ],
    [
      {
        x: 145,
        y: 115
      }
    ]
  ],
  result: 1
};
const inAcq = [
  {
    groupdata: [
      {
        variabledata: [
          {
            x: 145,
            y: 110
          }
        ],
        variablename: "counterS16"
      },
      {
        variabledata: [
          {
            x: 145,
            y: 111
          }
        ],
        variablename: "counterS8"
      },
      {
        variabledata: [
          {
            x: 145,
            y: 112
          }
        ],
        variablename: "counterU16"
      },
      {
        variabledata: [
          {
            x: 145,
            y: 113
          }
        ],
        variablename: "counterU8"
      },
      {
        variabledata: [
          {
            x: 145,
            y: 114
          }
        ],
        variablename: "varSLong"
      },
      {
        variabledata: [
          {
            x: 145,
            y: 115
          }
        ],
        variablename: "varSLongLong"
      }
    ],
    groupname: "Group_Var1"
  }
];

function config(param) {
  if ({}.hasOwnProperty.call(param, "stlinkRes")) {
    stlinkRs = param.stlinkRes;
  }
}

function connect() {
  var obj = this;
  const res = {
    MCUName: "NameOfMCU",
    result: stlinkRs
  };
  if (res.result === 1 /* STLINK_OK */) {
    obj.tout = setTimeout(function() {
      console.log(
        "Mock : One second has elapsed => AcqServ emits an Open event"
      );
      obj.emit("open", res.MCUName ? `(${res.MCUName})` : "");
    }, 1000);
  } else {
    obj.tout = setTimeout(function() {
      console.log(
        "Mock : One second has elapsed => AcqServ emits a Close event"
      );
      obj.emit("close", "Error STLINK NOK");
    }, 1000);
  }
}

function close(cb) {
  for (const groupname in this.groups) {
    if ({}.hasOwnProperty.call(this.groups, groupname)) {
      this.stopAcquisition(groupname, cb);
    }
  }
  cb();
  if (stlinkRs === 0) {
    this.emit("close");
    console.log("Mock : Emit Close");
  } else {
    setTimeout(function() {
      console.log("One second has elapsed => AcqServ emits a Close event");
      this.emit("close");
    }, 1000);
  }
}

function sendCommand(commandType, data, callback) {
  const mydata = data;
  switch (commandType) {
    case "read":
    case "READ":
      readMemory(this, mydata, callback, true);
      break;
    case "start":
    case "START":
      if (
        {}.hasOwnProperty.call(mydata, "groupname") &&
        {}.hasOwnProperty.call(mydata, "variablelist") &&
        {}.hasOwnProperty.call(mydata, "frequency") &&
        mydata.variablelist.length > 0
      ) {
        // if acquisition already started on this group for this probe, don't start a new one
        if (this.groups[mydata.groupname]) {
          return;
        }
        const variableslist = mydata.variablelist.map(variable => {
          variable.values = [];
          return variable;
        });
        const mygroup = {
          first: true,
          timerRead: {},
          variables: variableslist
        };
        this.groups[mydata.groupname] = mygroup;
        if (mydata.frequency === 0) {
          mygroup.timerRead = setImmediate(() => {
            this.readMemory(this, mydata, callback);
          });
        } else {
          mygroup.timerRead = setInterval(
            this.readMemory,
            1000 / mydata.frequency,
            this,
            mydata,
            callback
          );
        }
        if (!this.timerSend) {
          this.timerSend = setInterval(this.sendData, 50, this);
        }
      } else {
        callback("Parameter missing");
      }
      break;
    case "stop":
    case "STOP":
      this.stopAcquisition(mydata.groupname, callback);
      break;
    default:
      callback("Unknown command");
      break;
  }
}

function sendData(context) {
  const msg = [];
  for (const groupname in context.groups) {
    if (
      context.groups[groupname].variables.length > 0 &&
      context.groups[groupname].variables[0].values.length > 0
    ) {
      const item = {
        first: context.groups[groupname].first,
        groupdata: context.groups[groupname].variables.map(element => {
          return {
            variabledata: element.values,
            variablename: element.name
          };
        }),
        groupname
      };
      context.groups[groupname].first = false;
      msg.push(item);
      for (const variable of context.groups[groupname].variables) {
        variable.values = [];
      }
    }
  }
  context.emit("data", msg);
}

function readMemory(context, data, callback, oneshot) {
  let res;

  res = dataReceived;

  if (res.result === 1 /* STLINK_OK */ && res.data) {
    if (oneshot) {
      const respond = [
        {
          groupdata: res.data.map((element, i) => {
            return {
              variabledata: [element[0]],
              variablename: data.variablelist[i].name
            };
          }),
          groupname: data.groupname
        }
      ];
      console.log("Mock : Emit data");
      context.emit("data", respond);
    } else {
      /* not a oneshot read */
      for (let i = 0; i < res.data.length; i++) {
        context.groups[data.groupname].variables[i].values.push(res.data[i][0]);
      }
      if (data.frequency === 0) {
        context.groups[data.groupname].timerRead = setImmediate(() => {
          context.readMemory(context, data, callback);
        });
      }
    }
  }
}

function stopAcquisition(groupname, callback) {
  const group = this.groups[groupname];
  if (group) {
    if ({}.hasOwnProperty.call(group.timerRead, "_onImmediate")) {
      clearImmediate(group.timerRead);
    } else {
      clearInterval(group.timerRead);
    }
    delete this.groups[groupname];
    if (Object.keys(this.groups).length === 0) {
      clearInterval(this.timerSend);
      this.timerSend = null;
    }
  } else {
    callback("Unknown group");
  }
}

function setError(logMessage) {
  var obj = this;
  obj.tout = setTimeout(function() {
    obj.error(logMessage);
    console.log("One second has elapsed => emit Error");
  }, 1000);
}

module.exports.config = config;

module.exports.connect = connect;
module.exports.close = close;
module.exports.sendCommand = sendCommand;
module.exports.sendData = sendData;
module.exports.readMemory = readMemory;
module.exports.stopAcquisition = stopAcquisition;

module.exports.setError = setError;

module.exports.inAcq = inAcq;
