/*!
 * module node-red-contrib-stm32cubemonitor
 * Copyright(c) 2019 STMicroelectronics
 */
/* eslint-disable no-console */
var helper = require("node-red-node-test-helper");
var processing = require("../nodes/processing");
var variables = require("../nodes/variables");

helper.init(require.resolve("node-red"));

describe("processing Node", function() {
  before(function(done) {
    helper.startServer(done);
  });

  after(function(done) {
    helper.stopServer(done);
  });

  afterEach(function() {
    helper.unload();
  });

  it("should be loaded but variable node not present", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [],
        expressions: []
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.config.should.have.property("groupid", "n2");
      n1.should.have.property("id", "n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "red");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property(
          "text",
          "processing off - testname variable node not present or disable"
        );
        done();
      });
    });
  });

  it("should be loaded without error", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [],
        expressions: []
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.config.should.have.property("groupid", "n2");
      n1.should.have.property("id", "n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "green");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property("text", "processing on");
        done();
      });
    });
  });

  it("should indicate group variable is available but with other name", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [],
        expressions: []
      },
      {
        id: "n2",
        type: "variables",
        groupename: "othertestname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "yellow");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property(
          "text",
          "processing on but variable node name has been changed"
        );
        done();
      });
    });
  });

  it("should indicate ok with one expression", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [],
        expressions: [
          {
            index: "4",
            formula: "cos(variable1)",
            name: "cosinus"
          }
        ]
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "green");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property("text", "processing on");
        done();
      });
    });
  });

  it("should indicate expression is not more valid", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [],
        expressions: [
          {
            index: "4",
            formula: "cos(g_counter2)",
            name: "cosinus"
          }
        ]
      },
      {
        id: "n2",
        type: "variables",
        groupname: "othertestname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "red");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property(
          "text",
          "processing off - following outputs not valid : cosinus "
        );
        done();
      });
    });
  });

  it("should indicate ok with one statistic", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [
          {
            index: "3",
            name: "statmin",
            variable: "variable1",
            statistic: "min",
            scope: ""
          }
        ],
        expressions: []
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "green");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property("text", "processing on");
        done();
      });
    });
  });

  it("should indicate statistics is not more valid", function(done) {
    var flow = [
      {
        id: "n1",
        type: "processing",
        groupname: "testname",
        groupid: "n2",
        statistics: [
          {
            index: "3",
            name: "statmin",
            variable: "g_counter2",
            statistic: "min",
            scope: ""
          }
        ],
        expressions: []
      },
      {
        id: "n2",
        type: "variables",
        groupname: "othertestname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      }
    ];
    helper.load([processing, variables], flow, function() {
      var n1 = helper.getNode("n1");
      n1.computeData.on("message", msg => {
        msg.should.have.property("topic", "status");
        msg.status.should.have.property("fill", "red");
        msg.status.should.have.property("shape", "dot");
        msg.status.should.have.property(
          "text",
          "processing off - following outputs not valid : statmin "
        );
        done();
      });
    });
  });

  it("should format and send variable1 data to right output", function(done) {
    var flow = [
      {
        id: "n1",
        type: "variables",
        groupname: "testname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      },
      {
        id: "n2",
        type: "processing",
        groupname: "testname",
        groupid: "n1",
        statistics: [],
        expressions: [],
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load([processing, variables], flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        if (msg.topic == "data" && msg.payload.variablename == "variable1") {
          msg.payload.variabledata.should.be.an
            .Array()
            .and.containEql({ y: 10, x: 11 });
          done();
        }
      });
      n2.receive({
        payload: {
          data: [
            [
              {
                y: 10,
                x: 11
              }
            ],
            [
              {
                y: 12,
                x: 13
              }
            ]
          ],
          first: true,
          groupname: "testname"
        }
      });
    });
  });

  it("should format and send variable4 data to right output", function(done) {
    var flow = [
      {
        id: "n1",
        type: "variables",
        groupname: "testname",
        variablelist: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ]
      },
      {
        id: "n2",
        type: "processing",
        groupname: "testname",
        groupid: "n1",
        variables: [
          {
            name: "variable1",
            address: "0x2000000C",
            type: "1"
          },
          {
            name: "variable4",
            address: "0x20000010",
            type: "1"
          }
        ],
        statistics: [],
        expressions: [],
        outputs: 2,
        wires: [["n4"]]
      },
      { id: "n4", type: "helper" }
    ];
    helper.load([processing, variables], flow, function() {
      var n2 = helper.getNode("n2");
      var n4 = helper.getNode("n4");
      n4.on("input", function(msg) {
        if (msg.topic == "data" && msg.payload.variablename == "variable4") {
          msg.payload.variabledata.should.be.an
            .Array()
            .and.containEql({ y: 12, x: 13 });
          done();
        }
      });
      n2.receive({
        payload: {
          data: [
            [
              {
                y: 10,
                x: 11
              }
            ],
            [
              {
                y: 12,
                x: 13
              }
            ]
          ],
          first: true,
          groupname: "testname"
        }
      });
    });
  });
});
