/*!
 * module node-red-contrib-stm32cubemonitor
 * Copyright(c) 2019 STMicroelectronics
 */
// var assertions = require('./assertion');
// var should = require("should");
var helper = require("node-red-node-test-helper");
var variables = require("../nodes/variables");
// var mocha = require("mocha");
helper.init(require.resolve("node-red"));

describe("variables Node", function() {
  before(function(done) {
    helper.startServer(done);
  });

  after(function(done) {
    helper.stopServer(done);
  });

  afterEach(function() {
    helper.unload();
  });

  it("should be loaded", function(done) {
    var flow = [{ id: "n2", type: "variables", groupname: "testname" }];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      n2.should.have.property("groupname", "testname");
      done();
    });
  });

  it("should be loaded when exe-config is provided", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    var flow = [
      {
        id: "n1",
        name: "name1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      { id: "n2", type: "variables", groupname: "testname", execonfig: "n1" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      n2.should.have.property("groupname", "testname");
      n2.should.have.property("execonfig", "n1");
      done();
    });
  });

  it("should provide variables configuration with default values on payload  ", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: false
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: false
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        execonfig: "n1",
        frequencyType: "1",
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      //      var n1 = helper.getNode("n1");
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("direct");
        msg.payload.accesspoint.should.equal(0);
        msg.payload.frequency.should.equal(1);
        msg.payload.variablelist.should.be.a.Array().and.be.empty();
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with 3 manual variables and no imported as not selected in exe-config", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: false
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: false
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [
      { name: "variableM1", address: "0x20000000", type: 1 },
      { name: "variableM2", address: "0x20000002", type: 2 },
      { name: "variableM3", address: "0x20000004", type: 3 }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(manualVariableList[0])
          .and.containEql(manualVariableList[1])
          .and.containEql(manualVariableList[2]);
        msg.payload.variablelist.should.have.length(3);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with no manual variables and several imported from exe-config", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(exeConfigVariableList[1])
          .and.containEql(exeConfigVariableList[3]);
        msg.payload.variablelist.should.have.length(2);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with several manual variables and several imported from exe-config", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [
      { name: "variableM1", address: "0x20000000", type: 1 },
      { name: "variableM2", address: "0x20000002", type: 2 },
      { name: "variableM3", address: "0x20000004", type: 3 }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(manualVariableList[0])
          .and.containEql(manualVariableList[1])
          .and.containEql(manualVariableList[2])
          .and.containEql(exeConfigVariableList[1])
          .and.containEql(exeConfigVariableList[3]);
        msg.payload.variablelist.should.have.length(5);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with no duplicated variables", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [
      { name: "variableM1", address: "0x20000000", type: 1 },
      { name: "variable2", address: "0x20000002", type: 2 },
      { name: "variable3", address: "0x20000004", type: 3 }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(manualVariableList[0])
          .and.containEql(exeConfigVariableList[1])
          .and.containEql(exeConfigVariableList[3]);
        msg.payload.variablelist.should.have.length(3);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with trigger mode activated", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [
      { name: "variableM1", address: "0x20000000", type: 1 },
      { name: "variableM2", address: "0x20000002", type: 2 },
      { name: "variableM3", address: "0x20000004", type: 3 }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        triggerstartmode: "rising-edge",
        triggername: "variableM1",
        triggerthreshold: "5",
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.triggerstartmode.should.equal("rising-edge");
        msg.payload.triggername.should.equal("variableM1");
        msg.payload.triggerthreshold.should.equal("5");
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(manualVariableList[0])
          .and.containEql(manualVariableList[1])
          .and.containEql(manualVariableList[2])
          .and.containEql(exeConfigVariableList[1])
          .and.containEql(exeConfigVariableList[3]);
        msg.payload.variablelist.should.have.length(5);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with trigger mode de-activated because of unknown triggername", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [
      { name: "variableM1", address: "0x20000000", type: 1 },
      { name: "variableM2", address: "0x20000002", type: 2 },
      { name: "variableM3", address: "0x20000004", type: 3 }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        triggerstartmode: "rising-edge",
        triggername: "variable1",
        triggerthreshold: "5",
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.triggerstartmode.should.equal("manual");
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(manualVariableList[0])
          .and.containEql(manualVariableList[1])
          .and.containEql(manualVariableList[2])
          .and.containEql(exeConfigVariableList[1])
          .and.containEql(exeConfigVariableList[3]);
        msg.payload.variablelist.should.have.length(5);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });

  it("should provide payload with trigger mode de-activated because of erroneous threshold", function(done) {
    const exeConfigVariableList = [
      {
        address: "0x0800204c",
        name: "variable1",
        type: 1
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 2,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable2",
        type: 3
      },
      {
        address: "0x0800204c",
        name: "variable3",
        type: 4,
        checked: true
      },
      {
        address: "0x0800204c",
        name: "variable4",
        type: 8,
        checked: false
      }
    ];
    const manualVariableList = [
      { name: "variableM1", address: "0x20000000", type: 1 },
      { name: "variableM2", address: "0x20000002", type: 2 },
      { name: "variableM3", address: "0x20000004", type: 3 }
    ];
    const flow = [
      {
        id: "n1",
        type: "exe-config",
        exefile: "elf_file.txt",
        exevariablelist: exeConfigVariableList
      },
      {
        id: "n2",
        type: "variables",
        groupname: "testname",
        variablelist: manualVariableList,
        execonfig: "n1",
        frequencyType: "custom",
        frequency: "10.5",
        mode: "snapshot",
        accesspoint: 3,
        triggerstartmode: "rising-edge",
        triggername: "variable3",
        triggerthreshold: "",
        wires: [["n3"]]
      },
      { id: "n3", type: "helper" }
    ];
    helper.load(variables, flow, function() {
      var n2 = helper.getNode("n2");
      var n3 = helper.getNode("n3");
      n3.on("input", function(msg) {
        msg.should.have.property("topic", "start");
        msg.payload.groupname.should.equal("testname");
        msg.payload.mode.should.equal("snapshot");
        msg.payload.accesspoint.should.equal(3);
        msg.payload.frequency.should.equal(10.5);
        msg.payload.triggerstartmode.should.equal("manual");
        msg.payload.variablelist.should.be.an
          .Array()
          .and.containEql(manualVariableList[0])
          .and.containEql(manualVariableList[1])
          .and.containEql(manualVariableList[2])
          .and.containEql(exeConfigVariableList[1])
          .and.containEql(exeConfigVariableList[3]);
        msg.payload.variablelist.should.have.length(5);
        done();
      });
      n2.receive({ topic: "start" });
    });
  });
  // TODO: Add test for REST API
});

describe("exe-config Node", function() {
  before(function(done) {
    helper.startServer(done);
  });

  after(function(done) {
    helper.stopServer(done);
  });

  afterEach(function() {
    helper.unload();
  });

  it("should be loaded", function(done) {
    var flow = [{ id: "n1", type: "exe-config", exefile: "elf_file.txt" }];
    helper.load(variables, flow, function() {
      var n1 = helper.getNode("n1");
      n1.should.have.property("exefile", "elf_file.txt");
      done();
    });
  });
});
