/*!
 * module st-logger
 * Copyright(c) 2019 STMicroelectronics
 */
const expect = require("chai").expect;

/**
 * Tests Scenarios.
 */
describe("#getLogger()", function() {
  let logger;
  beforeEach(function() {
    logger = require("../src/logger");
  });

  afterEach(function() {
    delete require.cache[require.resolve("../src/logger")];
  });
  it("should work if the logger does not exist", function() {
    var myLogger = logger.getLogger("elfparser");
    expect(myLogger.level.levelStr).to.equal("INFO");
  });

  it("should work if the logger already exists", function() {
    var myLogger = logger.getLogger("elfparser");
    myLogger = logger.getLogger("elfparser");
    expect(myLogger.level.levelStr).to.equal("INFO");
  });

  it("should work if the log category is undefined", function() {
    var myLogger = logger.getLogger();
    expect(myLogger.level.levelStr).to.equal("INFO");
  });

  it("should work when reconfiguring to off logger level", function() {
    var myLogger = logger.getLogger("elfparser", "off");
    logger.updateLevel("off");
    expect(myLogger.level.levelStr).to.equal("OFF");
  });
  it("should work when reconfiguring to fatal logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("fatal");
    expect(myLogger.level.levelStr).to.equal("FATAL");
  });
  it("should work when reconfiguring to error logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("error");
    expect(myLogger.level.levelStr).to.equal("ERROR");
  });
  it("should work when reconfiguring to warn logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("warn");
    expect(myLogger.level.levelStr).to.equal("WARN");
  });
  it("should work when reconfiguring to info logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("info");
    expect(myLogger.level.levelStr).to.equal("INFO");
  });
  it("should work when reconfiguring to debug logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("debug");
    expect(myLogger.level.levelStr).to.equal("DEBUG");
  });
  it("should work when reconfiguring to trace logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("trace");
    expect(myLogger.level.levelStr).to.equal("TRACE");
  });
  it("should work when reconfiguring to unknown logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel("unknown");
    expect(myLogger.level.levelStr).to.equal("INFO");
  });
  it("should work when reconfiguring to unset logger level", function() {
    var myLogger = logger.getLogger("elfparser");
    logger.updateLevel();
    expect(myLogger.level.levelStr).to.equal("INFO");
  });
});
