/*!
 * module st-link-protocol
 * Copyright(c) 2019 STMicroelectronics
 */
/**
 * Memoryjs is used to read and write values at one address for one process.
 */
var ffi = require("ffi-napi");
var ref = require("ref-napi");
const path = require("path");

/**
 * Definition of types used on the DLL interface.
 */
var TAddress = ref.refType(ref.types.void);
var TValues = ref.refType(ref.types.void);

/**
 * DLL Loading + DLL interface definition.
 */

var platform = process.platform;
var memoryUtilities = null;
if (platform === "win32") {
  console.log("memoryUtilities = " + path.join(__dirname, "memoryUtilities.dll"));
  memoryUtilities = path.join(__dirname, "memoryUtilities.dll");
} else {
  throw new Error("unsupported plateform for stLinkUsbDriver");
}
try {
  var memoryHandler = ffi.Library(memoryUtilities, {
    writeBuffer: ["int", [ref.refType(TAddress), ref.refType(TValues), "int"]]
  });
} catch (err) {
  console.log(err);
}

/**
 * Service to write values (respValue) in the response buffer address (respAddr).
 * Note: Build the list of available "node" process at first call of writeValueInRespBuffer.
 *
 * @param {number[]} respAddr - Response buffer address.
 * @param {number[]} respValue - Array of bytes to write in the response buffer.
 */
function writeValueInRespBuffer(respAddr, respValue) {
  // console.log("respAddr = " + respAddr);
  // console.log("respValue = " + respValue);

  let myAddress = Buffer.from(respAddr);
  let myValues = Buffer.from(respValue);
  memoryHandler.writeBuffer(myAddress, myValues, myValues.length);
}

module.exports.writeValueInRespBuffer = writeValueInRespBuffer;
